// start3d.cpp
// by Glenn G. Chappell
// October 2003
//
// For CS 381
// A 3-D OpenGL/GLUT Program to Start With
//
// Draws a rotating square

#include <iostream>
using std::cout;
using std::cerr;
using std::endl;
#include <stdlib.h>  // Some versions of MS-Vis C++ have broken <cstdlib>
//using std::exit;
#include <GL/glut.h> // GLUT stuff - includes OpenGL headers as well


// Global variables
// Window/viewport
const int startwinsize = 400; // Starting window width & height, in pixels
const double neardist = 1.;   // Near & far clipping distances
const double fardist = 10.;

// Keyboard
const int ESCKEY = 27;        // ASCII value of escape character

// Scene
double angle = 0.;            // Rotation angle for object (degrees)
const double rotspeed = 1.;   // Rotation speed (degrees/frame)


// display
// The GLUT display function
void display()
{
   glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
   
   glPushMatrix();
      glTranslated(0., 0., -4.);  // Move object to where we can see it
      glRotated(angle, 0,1,0);

      glColor3d(0.9, 0.1, 0.1);
      glBegin(GL_QUADS);
         glVertex3d(-1., -1., 0.);  // Vertices in counterclockwise order
         glVertex3d( 1., -1., 0.);
         glVertex3d( 1.,  1., 0.);
         glVertex3d(-1.,  1., 0.);
      glEnd();
   glPopMatrix();

   glutSwapBuffers();
}


// reshape
// The GLUT reshape function
void reshape(int w, int h)
{
   glViewport(0, 0, w, h);

   glMatrixMode(GL_PROJECTION);
   glLoadIdentity();
   // We set up coordinate system so that aspect ratios are always correct,
   //  and the region from -1..1 in x & y always just fits in the viewport.
   if (w > h)
   {
      glFrustum(-double(w)/h, double(w)/h, -1., 1., neardist, fardist);
      // You may want to use gluPerspective here
   }
   else
   {
      glFrustum(-1., 1., -double(h)/w, double(h)/w, neardist, fardist);
   }

   glMatrixMode(GL_MODELVIEW);  // Always go back to modelview mode
}


// idle
// The GLUT idle function
void idle()
{
   // Print OpenGL errors, if there are any (for debugging)
   if (GLenum err = glGetError())
   {
      cerr << "OpenGL ERROR: " << gluErrorString(err) << endl;
   }

   // Update the scene
   angle += rotspeed;
   glutPostRedisplay();
}


// keyboard
// The GLUT keyboard function
void keyboard(unsigned char key, int x, int y)
{
   switch (key)
   {
   case ESCKEY:  // ESC: Quit
      exit(0);
      break;
   }
}


// init
// Initialization
// Called by main after window creation
void init()
{
   // General
   glClearColor(0.3, 0.4, 0.5, 0.0);  // Blue-gray background
   glEnable(GL_DEPTH_TEST);           // 3-D so we need HSR

   // Documentation
   cout << "\"Start\" 3-D Program" << endl;
   cout << "by Glenn G. Chappell" << endl;
   cout << "for CS 381, Fall 2003" << endl;
   cout << endl;
   cout << "Esc   Quit" << endl;
   cout << endl;
   cout << "Oh, no! The documentation is in the TEXT window!" << endl;
   cout << "(Better not do this in your homework.)" << endl;
}


int main(int argc, char ** argv)
{
   // Initialize OpenGL/GLUT
   glutInit(&argc, argv);
   glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGB | GLUT_DEPTH);

   // Make a window
   glutInitWindowSize(startwinsize, startwinsize);
   glutInitWindowPosition(50, 50);
   glutCreateWindow("CS 381 - \"Start\" 3-D Program");

   // Initialize GL states & register callbacks
   init();
   glutDisplayFunc(display);
   glutReshapeFunc(reshape);
   glutIdleFunc(idle);
   glutKeyboardFunc(keyboard);

   // Do something
   glutMainLoop();

   return 0;
}
